#! /usr/bin/env sh
MAXDOT=60
######################
##	Return Function	##
######################
SECTION()
{
	echo -e "\n\e[1;36m[$1]\e[0m"
}
ECHO()
{
	var="$2"
	len=$(($MAXDOT-${#var}))
	DOT=""

	for i in $(seq $len)
	do
		DOT="$DOT."
	done
	echo -n -e "$var $DOT \e[$1m["
	# -e will let \ as expression
	echo -n $3
	echo -e "]\e[0m"
}
SUCCESS()
{
	if [ -z "$2" ]; then
		ret="Pass"
	else
		ret="$2"
	fi
	ECHO "1;32" "$1" "$ret"
}
FAIL()
{
	if [ -z "$2" ]; then
		ret="Fail"
	else
		ret="$2"
	fi
	ECHO "1;31" "$1" "$ret"
}
MESSAGE()
{
	if [ -z "$2" ]; then
		ret="Not found"
	else
		ret="$2"
	fi
	ECHO "1;33" "$1" "$ret"
}
_GETKeyValue()
{
	key="$1"
	file="$2"
	var=`synogetkeyvalue $2 "$1"`
	echo "$var"
}
GetKeyValue()
{ 
	var=$(_GETKeyValue "$1" "$2")
	if [ -n "$3" ]; then
		$3 "$1" "$var"
	else
		if  [ -z "$var" ]; then
			FAIL "$1" "Not found"
		else
			SUCCESS "$1" "$var"
		fi
	fi
}
_PING()
{
	if [ "$1" != "PING" -a "$1" != "ALL" ]; then
		echo 1
		return 0
	fi
	local ip="$2"
	ret=`ping -c 1 -w 1 "$ip" | grep receive | cut -d ',' -f 2`
	if [ "1" == "${ret:1:1}" ]; then
		echo 1
	else
		echo 0
	fi
}
_BIND()
{
	if [ "$1" != "BIND" -a "$1" != "ALL" ]; then
		echo 1
		return 0
	fi
	local ip="$2"
	net ads auth -P -S $ip > /dev/null 2>&1
	if [ $? = 0 ]; then
		echo 1
	else
		echo 0
	fi
}
_KRB5()
{
	#Test with smbclient
	if [ "$1" != "KRB5" -a "$1" != "ALL" ]; then
		echo 1
		return 0
	fi
	local ip="$2"
	smbclient -P -L //${ip} -k > /dev/null 2>&1
	if [ $? = 0 ]; then
		echo 1
	else
		echo 0
	fi
}
_NTLM()
{
	#Test with smbclient
	if [ "$1" != "NTLM" -a "$1" != "ALL" ]; then
		echo 1
		return 0
	fi
	local ip="$2"
	smbclient -P -L //${ip} > /dev/null 2>&1
	if [ $? = 0 ]; then
		echo 1
	else
		echo 0
	fi
}
TEST()
{
	local Item="$1"
	if [ -n "$3" ]; then
		name="$3 [$2]"
	else
		name="$2"
	fi

	if [ "1" != $(_PING $1 $2) ]; then
		FAIL "$name" "Dead"
		return 0
	fi
	if [ "1" != $(_BIND $1 $2) ]; then
		FAIL "$name" "Bind Ldap Fail"
		return 0
	fi
	if [ "1" != $(_KRB5 $1 $2) ]; then
		FAIL "$name" "Auth krb5 Fail"
		return 0
	fi
	if [ "1" != $(_NTLM $1 $2) ]; then
		FAIL "$name" "Auth NTLM Fail"
		return 0
	fi
	SUCCESS "$name" "Alive"
}
##############
##	Testing	##
##############
DIG=`which dig`
if [ -f "/usr/syno/etc/smb.conf" ]; then
	SMB_CONF="/usr/syno/etc/smb.conf"
elif [ -f "/etc/samba/smb.conf" ]; then
	SMB_CONF="/etc/samba/smb.conf"
else
	FAIL "Can't find smb.conf"
	exit 1
fi
KERBEROS_RECORD="_kerberos._tcp"
LDAP_RECORD="_ldap._tcp"
hostname=`hostname`
domain=$(_GETKeyValue "security" $SMB_CONF)
realm=`grep "realm" $SMB_CONF | cut -d '=' -f 2`
workgroup=`grep "workgroup" $SMB_CONF | cut -d '=' -f 2`

## Check DS is already join Domain
security=`cat $SMB_CONF | grep security | grep user`
if [ -n "$security" ]; then
	FAIL "We only check the DS already join Domain"
	exit 1
fi

SECTION "config"
GetKeyValue "security" $SMB_CONF
GetKeyValue "admin users" $SMB_CONF
GetKeyValue "workgroup" $SMB_CONF
GetKeyValue "realm" $SMB_CONF
GetKeyValue "password server" $SMB_CONF MESSAGE

##########################
##	Check the time sync	##
##########################
{
	ret=`net ads info 2>/dev/null | grep Server_time_offset | cut -d '=' -f 2 | cut -d '"' -f 2`
	if [ -z "$ret" ]; then
		FAIL "Sync Time" "Cannot get Sync time"
	fi

	if [ "600" -le "$ret" ]; then
		FAIL "Sync Time" "Sync Time Too Large $ret"
	else
		SUCCESS "Sync Time" "Pass"
	fi

}

##############################
##	Check duplicate DS name	##
##############################
{
	SUCCESS "hostname" "$hostname"
	if [ "$hostname" == "$workgroup" ]; then
		FAIL "hostname check" "Conflict with realm"
	fi

	nr=`dsassistant 2>&1 | grep "$hostname" | wc -l`
	if [ "$nr" -ge 2 ]; then
		FAIL "In LAN we have $n $hostname"
	fi
}


##################
##	DNS related	##
##################
SECTION "Test DC by DNS"
if [ -n "$realm" ]; then
	ret=`nslookup $realm 2>&1 | tail -n +5`
	if [ -z "$ret" ]; then
		FAIL "nslookup $realm"
	else
		nslookup $realm 2>&1 | tail -n +5 | grep Address | while read line
		do
			ip=`echo $line | cut -d ':' -f 2 | cut -d ' ' -f 2`
			TEST "ALL" $ip
		done
	fi
fi

######################
##	DIG operations	##
######################
if [ -f "$DIG" ]; then
	SECTION "Test DC by dig"
	{
		$DIG ns $realm | grep "\sA\s" | while read LINE
		do
			_domain_=`echo $LINE | cut -d ' ' -f 1`
			_ip_=`echo $LINE | cut -d ' ' -f 5`
			TEST "ALL" $_ip_ $_domain_
		done
	}

	SECTION "Test kerberos by dig"
	{
		$DIG srv $KERBEROS_RECORD.$realm | grep "\sA\s" | while read LINE
		do
			_domain_=`echo $LINE | cut -d ' ' -f 1`
			_ip_=`echo $LINE | cut -d ' ' -f 5`
			TEST "PING" $_ip_ $_domain_
		done
	}

	SECTION "Ping ldap by dig"
	{
		$DIG srv $LDAP_RECORD.$realm | grep "\sA\s" | while read LINE
		do
			_domain_=`echo $LINE | cut -d ' ' -f 1`
			_ip_=`echo $LINE | cut -d ' ' -f 5`
			TEST "BIND" $_ip_ $_domain_
		done
	}
fi
